#|
                                                            
 Title    : 				SD_Common_Functions.lsp                         
 Purpose  : 				Functions for OneSpace Designer             
 Created  : 				8 May 2003                                      
 Modified :                                                 
 Language : 				Lisp                                            
 Author   : 				R. Weimar                                       
                                                            
 Comment  : 				Uses the file c:\OSD Customization\SD_Common_Customize.ini for storing settings
|#

(in-package 'Common)
(use-package :oli)

(sd-defdialog 'Create_Hole
	:dialog-title "Hole"
  :variables
  '(
		(FeatureFace 			:value-type :face
											:incl-position :3d
											:modifies :contents
											:multiple-items nil
											:title "Face"
											:prompt-text "Pick Face"
											:after-input 
											(progn			
												(setq Owner (sd-inq-parent-obj (first FeatureFace)))
											)
		);FeatureFace
		("Chamfer parameters")
	 	(Chamfer							:value-type	:number
			 	 									:initial-value 	0
		);chamfer
		(ChamferAngle					:title "Angle"
													:value-type	:number
                 					:proposals ("5" "10" "15" "20" "30" "45")
													:initial-value 45
													
		);ChamferAngle
		(Chamferbothsides			:value-type :boolean
													:toggle-type :wide-toggle
													:title "Chamfer both sides"
													:initial-value t
													:initial-visible nil
													
	 );Chamferbothsides
	 (diameter							:value-type	:number
			 	 									:initial-value 	10
	 )
	)
  :ok-action
  '(progn
  	(setq normal (sd-inq-face-pnt (first FeatureFace)
																		 :u (gpnt2d_x (second Featureface))
																		 :v (gpnt2d_y (second Featureface))
																		 :all t
																		 :dest-space :global))
		(setq coordinates (getf normal :coordinates))
		(setq normal (getf normal :normal))
		(sd-call-cmds (create_workplane :new
																		:pt_dir :origin
																		coordinates
																		:normal
																		normal
									)	
		);sd-call-cmds
		(sd-call-cmds (CIRCLE :CEN_RAD 0,0 (/ Diameter 2.0)))
		(sd-call-cmds	(punch 	:parts owner :keep_wp :yes :keep_profile :no))(setq facelist (sd-call-cmds (get_selection :focus_type *sd-cylinder-seltype* :select :selected_part owner :all_3d)))
		(dolist (j facelist)
			(setf Origin (sd-cylinder-center (sd-inq-geo-props j :dest-space (sd-inq-curr-wp))))
			(if (and (< (ABS (gpnt3d_x Origin)) 0.00001) (< (ABS (gpnt3d_y Origin)) 0.00001))
				(progn
					(setf Changed_face j)											
					(return)
				);progn
			);if
		);dolist
		(setq edgelist (sd-call-cmds (get_selection :focus_type *sd-edge-3d-seltype* :select :by_face Changed_face)))
		(dolist (i edgelist)
			(setf CenterCircle nil)
			(if (sd-circle-p (sd-inq-geo-props i :dest-space (sd-inq-curr-wp)))
				(setf CenterCircle (sd-circle-center (sd-inq-geo-props i :dest-space (sd-inq-curr-wp))))
			);if
			(if (sd-ellipse-p (sd-inq-geo-props i :dest-space (sd-inq-curr-wp)))
				(setf CenterCircle (sd-ellipse-center (sd-inq-geo-props i :dest-space (sd-inq-curr-wp))))
			);if
			(if centercircle
				(if (or (< (ABS (gpnt3d_z CenterCircle)) 0.00001) chamferbothsides)
					(sd-call-cmds (chamfer 	:blend_type :C_DIST_ANGLE :edge_3d i :new_angle_dist chamfer (* (/ chamferangle 180) PI)))
				);if
			);if
		);dolist
		(if (/= chamferangle 45)
			(progn
				(sd-call-cmds (position_wp (sd-inq-curr-wp) :rotate :axis :u :rotation_angle (/ PI 2) :done))
				(setq edgelist (sd-call-cmds (get_selection :focus_type *sd-edge-3d-seltype* :select :by_face Changed_face)))
				(dolist (j edgelist)
					(setf Selchamfer (sd-call-cmds (get_selection :focus_type *sd-chamfer-3d-seltype* :select :by_edge_3d j)))
					(sd-call-cmds (uic_project_face :face_3d Selchamfer))
					(setf linelist (sd-call-cmds (get_selection :focus_type *sd-line-2d-seltype* :select :all_2d)))
					(setf line1 (first linelist))
					(pop linelist)
					(setf ChamferOK nil)
					(dolist (i linelist)
						(setf measureangle (/ (* 180 (sd-call-cmds (measure_angle :edge  line1 i))) PI))
						(if (>= measureangle 90)
							(setf measureangle (- 180 measureangle))
						);if
						(if (< (abs (- 90 (+ chamferangle measureangle))) 0.00001)
							(progn
								(setf ChamferOK t)																														
								(return)
							);progn
						);if
					);dolist
					(if (not ChamferOK)
						(sd-call-cmds (modify_chamfers :new_selection selchamfer :blend_type :C_DIST_ANGLE :new_angle_dist chamfer (* (/ chamferangle 180) PI) :da_reverse))
					)
					(sd-call-cmds (delete_2d :all_2d))
				);dolist
			);progn
		);if
		(sd-call-cmds (delete_3d (sd-inq-curr-wp)))
	)	
)

